/* Remote Control for CL-Amp
 *
 * clr.cpp
 *
 * (C) 1999 by Mikael Jansson
 *
 * v5
*/

#include <iostream.h>
#include <stdiostream.h>
#include <Application.h>
#include <stdio.h>
#include <string.h>
#include "FrontEnd.h"
#include "option.h"

#include "interface/Window.h"  // Be's BWindow class
#include "interface/View.h" 
#include "interface/TextControl.h"
#include "interface/Button.h"
#include "interface/ListView.h"
#include "interface/ScrollView.h"
#include "interface/ListItem.h"
#include "be/support/List.h"
#include <String.h>

const uint32 CANCEL_BTN_PRESSED = 'DoIt';
const uint32 LIST_ITEM_SELECTED = 'FdIt';
const uint32 TEXT_CONTROL_MODIFIED = 'FdTx';
const uint32 TEXT_CONTROL_ENTERED = 'EtTx';

// create a new CL-Amp object.
CLAmp_FrontEnd Remote;
CLAmpSongEntry E;

struct SongObj
{
	long TrackNum;
	BString* TrackPathAndName;
};

BList *SongObjList = new BList();

bool fade;

//custom BWindow subclass
class BrentWindow : public BWindow
{
	public:
		BrentWindow();
		virtual bool QuitRequested();
		virtual void MessageReceived(BMessage* msg);
		SongObj *mySOItem;
		BButton* btnCancel;
		BListView* lstFound;
		BTextControl*  txtInput;
		BScrollView* lstVScrollView;
};

//function implementation of BrentWindow() (contructor);
BrentWindow :: BrentWindow()
/*
	: BWindow(BRect(175.000000,400.000000,565.000000,800.000000), "Jump to file", B_TITLED_WINDOW,B_NOT_RESIZABLE|B_NOT_ZOOMABLE)
*/
	: BWindow(BRect(200.000000,200.000000,590.000000,600.000000), "Jump to file", B_TITLED_WINDOW,B_NOT_RESIZABLE|B_NOT_ZOOMABLE)

{
	SetLook(B_FLOATING_WINDOW_LOOK);
	BView* myview = new BView(Bounds(),NULL,B_FOLLOW_ALL_SIDES,0);
	myview->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	AddChild(myview);
		
	BRect txtControlBounds(7.000000,10.000000,385.000000,30.000000);
	txtInput = new BTextControl(txtControlBounds,"txtInput",NULL,"Search for text",NULL);
	txtInput->SetModificationMessage(new BMessage(TEXT_CONTROL_ENTERED));
	
	myview->AddChild(txtInput);
	txtInput->MakeFocus();
	
	lstFound = new BListView(BRect(10,40,370,350),"lstFound");
	lstFound->SetViewColor(60,0,60,0);
	lstFound->SetHighColor(255,255,0,0);
	lstFound->SetLowColor(60,0,60,0);
	
	lstFound->SetInvocationMessage(new BMessage(LIST_ITEM_SELECTED));
	lstFound->AttachedToWindow();

	lstVScrollView = new BScrollView("ScrollFoundSongs",lstFound,B_FOLLOW_LEFT | B_FOLLOW_TOP, 0, false, true);
	myview->AddChild(lstVScrollView);
	lstVScrollView->AttachedToWindow();

	BButton* btnCancel = new BButton(BRect(10,360,385,390), "btnCancel", "Cancel", new BMessage(CANCEL_BTN_PRESSED));
	myview->AddChild(btnCancel);
	
/*
***	Add all the Tracks in the list and scroll to the one that's currently playing (or active)
*/
	for (int L=0; Remote.GetEntry(&E, L); ++L)
	{
		lstFound->AddItem(new BStringItem((char* )E.GetTitle()));
		//lstFound->AddItem(new BStringItem((char* )E.GetFileName()));
		mySOItem = new SongObj;
		mySOItem->TrackNum =  E.IndexNr()+1;
		//mySOItem->TrackPathAndName = new BString(E.GetFileName());
		SongObjList->AddItem(mySOItem);
	 }
	
	//mySOItem = (SongObj *)SongObjList->ItemAt(200);
	//printf("After list full ItemAt(200) %d,%s\n",mySOItem->TrackNum,mySOItem->TrackPathAndName->String());
	
	lstFound->Select(Remote.GetTitleNum());
	lstFound->ScrollToSelection();

}

// Called when the user clicks on the window's close box
bool BrentWindow :: QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);// tell the application to quit too
	return true; // and indicate that it's okay for us to close
}
void BrentWindow::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
		case TEXT_CONTROL_ENTERED:

			BString *TextToFindBString;
			TextToFindBString = new BString(txtInput->Text());

			if (TextToFindBString->Length()<1)
			{
				printf("leaving\n");
				break;
			}
			
			int32 iIndex;
			BString *FoundItemBString;
	
			lstFound->MakeEmpty();
			SongObjList->MakeEmpty();
			
			for (int L=0; Remote.GetEntry(&E, L); ++L)
			{
				//FoundItemBString = new BString(E.GetTitle());
				FoundItemBString = new BString(E.GetFileName());

				iIndex = FoundItemBString->IFindFirst(txtInput->Text());

				if (iIndex >= 0)
				{
					
					lstFound->AddItem(new BStringItem((char* )E.GetTitle()));
					mySOItem = new SongObj;
					mySOItem->TrackNum =  E.IndexNr()+1;
					//mySOItem->TrackName = new BString(E.GetTitle());
					//mySOItem->TrackPathAndName = new BString(E.GetFileName());
					SongObjList->AddItem(mySOItem);
				}
				
			}
			
		break;
		
		case CANCEL_BTN_PRESSED:
			
			be_app->PostMessage(B_QUIT_REQUESTED);

		break;
		
		case LIST_ITEM_SELECTED:
			
			mySOItem = (SongObj *)SongObjList->ItemAt(lstFound->CurrentSelection(0));

				Remote.PlayThis(mySOItem->TrackNum-1,fade);
				//printf("Playthis %d,%s\n",mySOItem->TrackNum,mySOItem->TrackName->String());

				be_app->PostMessage(B_QUIT_REQUESTED);

		break;

		default:
			BWindow::MessageReceived(msg);
		break;
	}
}

int main( int argc, char **argv )
{
#define SNOOZE_TIME (100 * 1000)
#define MAX_TIME (5 * 1000 * 1000)
#define MAX_LOOPS (MAX_TIME / SNOOZE_TIME)

	long Cnt=0;

	if (!Remote.InitCheck()) {
		printf( "clr: error initializing\n");	
		return 1;
	}
	Remote.StartRunning( true, B_NORMAL_PRIORITY );
	while (!Remote.Refresh() && Cnt++<MAX_LOOPS) snooze(SNOOZE_TIME);
	if (Cnt>=MAX_LOOPS) {
		printf( "clr: error, no answer\n");	
		return 1;
	}
	
	fade = false;
	
	
	if (argc>1)
	{
		if (strcmp(argv[1], "-fade") == 0) {
		//printf("clr %s\nfor more info check out http://home2.sbbs2.com/tic_khr\n", revision);
		//return 0;
		printf("fade is on\n");
		fade = true;
		}
	}
	
	BApplication app("application/x-vnd.JumpToFile-CL-Amp");

	BWindow* window = new BrentWindow();

	window->Show();
	app.Run();

	return 0;
}